@@ -1,6 +1,6 @@
 package CodeGen::Cpppp::Template;
 
-our $VERSION = '0.003'; # VERSION
+our $VERSION = '0.004'; # VERSION
 # ABSTRACT: Base class for template classes created by compiling cpppp
 
 use v5.20;
@@ -21,15 +21,21 @@ package CodeGen::Cpppp::Template::Exports {
       PROTECTED  => 'protected',
       PRIVATE    => 'private',
    };
-   our @EXPORT_OK= qw( PUBLIC PROTECTED PRIVATE compile_cpppp );
-   our %EXPORT_TAGS= (
-      'v0' => \@EXPORT_OK,
+   our @EXPORT_OK= qw( PUBLIC PROTECTED PRIVATE compile_cpppp format_commandline
+     format_timestamp
    );
+   our %EXPORT_TAGS= (
+      'v0' => [qw( PUBLIC PROTECTED PRIVATE compile_cpppp )],
+   );
+   #sub util {
+   #   return bless [ caller ], __PACKAGE__;
+   #}
+   #sub _caller { ref $_[0] eq __PACKAGE__? @{+shift} : caller(1) }
    sub compile_cpppp {
       my ($pkg, $filename, $line)= caller;
       my $cpppp;
       if (@_ == 1) {
-         # If the argument is multiple lines, assume it is cpppp code
+         # If the argument has any line terminator, assume it is cpppp code
          if (index($_[0], "\n") >= 0) {
             $cpppp= $_[0];
          }
@@ -49,7 +55,7 @@ package CodeGen::Cpppp::Template::Exports {
             close $fh;
          }
       }
-      Carp::croak("parse_cppp argument should either be '__DATA__' or lines of cpppp code ending with '\\n'")
+      Carp::croak("compile_cppp argument should either be '__DATA__' or lines of cpppp code ending with '\\n'")
          unless defined $cpppp;
       Carp::croak("cpppp source cannot be empty")
          unless length $cpppp;
@@ -59,6 +65,14 @@ package CodeGen::Cpppp::Template::Exports {
       $pkg->_build_BUILD_method(
          $pkg->cpppp_version, $parse->{code}, $filename, $line);
    }
+   sub format_commandline {
+      return '' unless main->can('format_commandline');
+      return main->format_commandline;
+   }
+   sub format_timestamp {
+      my @t= gmtime;
+      sprintf "%04d-%02d-%02dT%02d:%02d:%02dZ", $t[5]+1900, @t[4,3,2,1,0]
+   }
 }
 
 sub _tag_for_version($ver) {
@@ -178,6 +192,8 @@ sub autocolumn        { $_[0]{autocolumn}       = $_[1]||0 if @_ > 1; $_[0]{auto
 sub autocomma         { $_[0]{autocomma}        = $_[1]||0 if @_ > 1; $_[0]{autocomma}         }
 sub autoindent        { $_[0]{autoindent}       = $_[1]||0 if @_ > 1; $_[0]{autoindent}        }
 sub autostatementline { $_[0]{autostatementline}= $_[1]||0 if @_ > 1; $_[0]{autostatementline} }
+sub indent            { $_[0]{indent}           = $_[1]    if @_ > 1; $_[0]{indent} }
+sub emit_POD          { $_[0]{emit_POD}         = $_[1]||0 if @_ > 1; $_[0]{emit_POD} }
 
 sub _parse_data($class) {
    $class = ref $class if ref $class;
@@ -210,6 +226,7 @@ sub new($class, @args) {
       (map +($_ => $parse->{$_}||0), qw(
          autoindent autocolumn convert_linecomment_to_c89
       )),
+      indent => $parse->{indent},
       output => CodeGen::Cpppp::Output->new,
       current_output_section => 'private',
       %attrs,
@@ -277,6 +294,13 @@ sub define_template_method($self, $name, $code) {
    $self->{template_method}{$name}= $code;
 }
 
+sub _render_pod_block($self, $i) {
+   if ($self->emit_POD) {
+      $self->_finish_render;
+      $self->{output}->append($self->{current_output_section} => $self->_parse_data->{pod_blocks}[$i]);
+   }
+}
+
 sub _finish_render($self) {
    return unless defined $self->{current_out};
    # Second pass, adjust whitespace of all column markers so they line up.
@@ -331,6 +355,17 @@ sub _render_code_block {
       elsif (defined $s->{eval_idx}) {
          my $fn= $expr_subs[$s->{eval_idx}]
             or die;
+         # Identify the indent settings at this point so that other modules can
+         # automatically generate matching code.
+         my ($last_char)= ($$out =~ /(\S) (\s*) \Z/x);
+         my $cur_line= substr($$out, rindex($$out, "\n")+1);
+         (my $indent_prefix= $cur_line) =~ s/\S/ /g;
+         local $CodeGen::Cpppp::CURRENT_INDENT_PREFIX= $indent_prefix;
+         local $CodeGen::Cpppp::INDENT= $self->indent if defined $self->indent;
+         # it is "inline" context if non-whitespace occurs on this line already
+         my $is_inline= !!($cur_line =~ /\S/);
+         local $CodeGen::Cpppp::CURRENT_IS_INLINE= $is_inline;
+
          # Avoid using $_ up to this point so that $_ pases through
          # from the surrounding code into the evals
          my @out= $fn->($self, $out);
@@ -338,15 +373,9 @@ sub _render_code_block {
          @out= @{$out[0]} if @out == 1 && ref $out[0] eq 'ARRAY';
          ref eq 'CODE' && ($_= $_->($self, $out)) for @out;
          @out= grep defined, @out;
-         # Now decide what to join them with.
+         # Now decide how to join this into the code template.
+         # If this interpolation does not occur at the beginning of the line,
          my $join_sep= $";
-         my $indent= '';
-         my ($last_char)= ($$out =~ /(\S) (\s*) \Z/x);
-         my $cur_line= substr($$out, rindex($$out, "\n")+1);
-         my $inline= $cur_line =~ /\S/;
-         if ($self->{autoindent}) {
-            ($indent= $cur_line) =~ s/\S/ /g;
-         }
          # Special handling if the user requested a list substitution
          if (ord $s->{eval} == ord '@') {
             $last_char= '' unless defined $last_char;
@@ -354,7 +383,7 @@ sub _render_code_block {
                && substr($text, $s->{pos}+$s->{len}, 1) eq ';'
             ) {
                @out= grep /\S/, @out; # remove items that are only whitespace
-               if (!$inline && substr($text, $s->{pos}+$s->{len}, 2) eq ";\n") {
+               if (!$is_inline && substr($text, $s->{pos}+$s->{len}, 2) eq ";\n") {
                   $join_sep= ";\n";
                   # If no elements, remove the whole line.
                   if (!@out) {
@@ -367,14 +396,14 @@ sub _render_code_block {
             }
             elsif ($self->{autocomma} && ($last_char eq ',' || $last_char eq '(' || $last_char eq '{')) {
                @out= grep /\S/, @out; # remove items that are only whitespace
-               $join_sep= $inline? ', ' : ",\n";
+               $join_sep= $is_inline? ', ' : ",\n";
                # If no items, or the first nonwhitespace character is a comma,
                # remove the previous comma
                if (!@out || $out[0] =~ /^\s*,/) {
                   $$out =~ s/,(\s*)\Z/$1/;
                }
             }
-            elsif ($self->{autoindent} && !$inline && $join_sep !~ /\n/) {
+            elsif ($self->{autoindent} && !$is_inline && $join_sep !~ /\n/) {
                $join_sep .= "\n";
             }
          }
@@ -383,8 +412,8 @@ sub _render_code_block {
             my $str= shift @out;
             $str .= $join_sep . $_ for @out;
             # Autoindent: if new text contains newline, add current indent to start of each line.
-            if ($self->{autoindent} && $indent) {
-               $str =~ s/\n/\n$indent/g;
+            if ($self->{autoindent} && length $indent_prefix) {
+               $str =~ s/\n/\n$indent_prefix/g;
             }
             $$out .= $str;
          }
@@ -422,7 +451,7 @@ and postderef.
 
 =item C<:v0>
 
-Exports symbols C<PUBLIC>, C<PROTECTED>, C<PRIVATE>
+Exports symbols C<PUBLIC>, C<PROTECTED>, C<PRIVATE>, C<compile_cpppp>
 
 =back
 
@@ -461,6 +490,24 @@ that contains a newline.
 Whether to automatically insert newlines (and maybe indent) when substituting
 an array into a template, based on context.
 
+=head2 indent
+
+The per-block indent to use for generated code.  This can be set to either a
+number (of spaces) or a literal string to be appended for each level of indent.
+If undefined, the indent will be detected from the change in leading whitespace
+from the first observed '{' in your template.
+
+This setting does not re-format the existing indentation written in the
+template; you need a full code-formatting tool for that.
+
+=head2 emit_POD
+
+By default, Plain Old Documentation (POD) notation found in the template is
+assumed to document the template itself, and will be removed from the generated
+output.  Set this to true to emit the POD as part of the output.  (but a better
+idea is to declare variables like C<< $head1= '=head1' >> and then use those
+to generate the output POD)
+
 =head1 CONSTRUCTOR
 
   $tpl= $template_class->new(%params, %attrs);
@@ -511,11 +558,11 @@ Michael Conrad <mike@nrdvana.net>
 
 =head1 VERSION
 
-version 0.003
+version 0.004
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2023 by Michael Conrad.
+This software is copyright (c) 2024 by Michael Conrad.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
